@@ -81,6 +81,43 @@ my $VERSIONED_INTERPRETERS
   = Lintian::Data->new('scripts/versioned-interpreters',
     qr/\s*=\>\s*/o,\&_parse_versioned_interpreters);
 
+# When detecting commands inside shell scripts, use this regex to match the
+# beginning of the command rather than checking whether the command is at the
+# beginning of a line.
+my $LEADINSTR = '(?:(?:^|[`&;(|{])\s*|(?:if|then|do|while)\s+)';
+my $LEADIN = qr/$LEADINSTR/;
+
+#forbidden command in maintainer scripts
+my $BAD_MAINT_CMD = Lintian::Data->new(
+    'scripts/maintainer-script-bad-command',
+    qr/\s*\~\~/,
+    sub {
+        my @sliptline = split(/\s*\~\~/, $_[1], 4);
+        if(scalar(@sliptline) != 4) {
+            fail 'Syntax error in scripts/maintainer-script-bad-command:', $.;
+        }
+        my ($incat,$exceptinpackage,$inscript,$regexp) = @sliptline;
+        $regexp =~ s/\${LEADIN}/$LEADINSTR/;
+   # allow empty $exceptinpackage and set it synonymous to check in all package
+        $exceptinpackage
+          = defined($exceptinpackage) ? strip($exceptinpackage) : '';
+        if (length($exceptinpackage) == 0) {
+            $exceptinpackage = '\a\Z';
+        }
+        # allow empty $inscript and set to synonymous to check in all script
+        $inscript = defined($inscript) ? strip($inscript) : '';
+        if (length($inscript) == 0) {
+            $inscript = '.*';
+        }
+        return {
+            # use not not to normalize boolean
+            'in_cat_string' => not(not(strip($incat))),
+            'in_package' => qr/$exceptinpackage/x,
+            'in_script' => qr/$inscript/x,
+            'regexp' => qr/$regexp/x,
+        };
+    });
+
 # Any of the following packages can satisfy an update-inetd dependency.
 my $update_inetd = join(
     ' | ', qw(update-inetd inet-superserver openbsd-inetd
@@ -99,11 +136,6 @@ my @depends_needed = (
     ['xml-core'    => '\bupdate-xmlcatalog\s'],
 );
 
-# When detecting commands inside shell scripts, use this regex to match the
-# beginning of the command rather than checking whether the command is at the
-# beginning of a line.
-my $LEADIN = qr'(?:(?:^|[`&;(|{])\s*|(?:if|then|do|while)\s+)';
-
 my @bashism_single_quote_regexs = (
     $LEADIN . qr'echo\s+(?:-[^e\s]+\s+)?\'[^\']*(\\[abcEfnrtv0])+.*?[\']',
     # unsafe echo with backslashes
@@ -783,80 +815,25 @@ sub run {
                     }
                 }
                 if (!$cat_string) {
-                    if (/^\s*start-stop-daemon\s+/ && !/\s--stop\b/) {
-                        tag 'start-stop-daemon-in-maintainer-script',
-                          "$file:$.";
-                    }
-                    # Don't use chown foo.bar
-                    if (
-                        m{(chown(?:\s+--?[A-Za-z-]+)*\s+
-                          [-_A-Za-z0-9]+\.[-_A-Za-z0-9]+)\s+
-                         }xsm
-                      ) {
-                        tag 'deprecated-chown-usage', "$file:$. \'$1\'";
-                    }
-                    if (/invoke-rc.d.*\|\| exit 0/) {
-                        tag 'maintainer-script-hides-init-failure', "$file:$.";
-                    }
+                    generic_check_bad_command($_, $file, $., $pkg, 0);
+
                     if (m,/usr/share/debconf/confmodule,) {
                         $saw_debconf = 1;
                     }
                     if (m/^\s*read(?:\s|\z)/ && !$saw_debconf) {
                         tag 'read-in-maintainer-script', "$file:$.";
                     }
-                    if (m,^\s*rm\s+(?:[^>]*\s)?/dev/([^/ ]+),) {
-                        my $f = $1//'';
-                        if ($f ne 'shm' && $f !~ m,^\.[^.]+,) {
-                            # Ignore /dev/shm (initscripts) and
-                            # dot-files (which are probably not a
-                            # dev).
-                            tag 'maintainer-script-removes-device-files',
-                              "$file:$.";
-                        }
-                    }
-                    if (m,>\s*(/etc/(?:services|protocols|rpc))(?:\s|\Z),) {
-                        tag 'maintainer-script-modifies-netbase-managed-file',
-                          "$file:$. $1";
-                    }
-                    if (
-                        m{\A \s*(?:cp|mv) \s .* 
-                                (/etc/(?:services|protocols|rpc))
-                         \s*\Z}xsm
-                      ) {
-                        tag 'maintainer-script-modifies-netbase-managed-file',
-                          "$file:$. $1";
-                    }
+
                     if (m,>\s*/etc/inetd\.conf(?:\s|\Z),) {
-                        tag 'maintainer-script-modifies-inetd-conf', "$file:$."
+                        tag 'maintainer-script-modifies-inetd-conf',"$file:$."
                           unless $info->relation('provides')
                           ->implies('inet-superserver');
                     }
                     if (m,^\s*(?:cp|mv)\s+(?:.*\s)?/etc/inetd\.conf\s*$,) {
-                        tag 'maintainer-script-modifies-inetd-conf', "$file:$."
+                        tag 'maintainer-script-modifies-inetd-conf',"$file:$."
                           unless $info->relation('provides')
                           ->implies('inet-superserver');
                     }
-                    if (m,>\s*/etc/ld\.so\.conf(?:\s|\Z),) {
-                        tag 'maintainer-script-modifies-ld-so-conf', "$file:$."
-                          unless $pkg =~ /^libc/;
-                    }
-                    if (m,^\s*(?:cp|mv)\s+(?:.*\s)?/etc/ld\.so\.conf\s*$,) {
-                        tag 'maintainer-script-modifies-ld-so-conf', "$file:$."
-                          unless $pkg =~ /^libc/;
-                    }
-
-                    # Ancient dpkg feature tests.
-                    if (m/${LEADIN}dpkg\s+--assert-working-epoch\b/) {
-                        tag 'ancient-dpkg-epoch-check', "$file:$.";
-                    }
-                    if (m/${LEADIN}dpkg\s+--assert-multi-conrep\b/) {
-                        tag 'ancient-dpkg-multi-conrep-check', "$file:$.";
-                    }
-
-                    # Commands that should not be used in maintainer scripts.
-                    if (m,${LEADIN}(?:/usr/bin/)?fc-cache(?:\s|\Z),) {
-                        tag 'fc-cache-used-in-maintainer-script', "$file:$.";
-                    }
 
                     # Check for running commands with a leading path.
                     #
@@ -888,14 +865,7 @@ sub run {
                     }
                 }
             }
-            if (m,\bsuidregister\b,) {
-                tag 'suidregister-used-in-maintainer-script', $file;
-            }
-            if ($file eq 'postrm') {
-                if (m,update\-alternatives \-\-remove,) {
-                    tag 'update-alternatives-remove-called-in-postrm';
-                }
-            } else {
+            unless ($file eq 'postrm') {
                 for my $rule (@depends_needed) {
                     my ($package, $regex) = @$rule;
                     if (    $pkg ne $package
@@ -920,22 +890,9 @@ sub run {
                     }
                 }
             }
-            if (m,\bgconftool(?:-2)?(?:\s|\Z),) {
-                tag 'gconftool-used-in-maintainer-script', "$file:$.";
-            }
-            if (m,\binstall-sgmlcatalog\b,
-                && !(m,--remove, && ($file eq 'prerm' || $file eq 'postinst')))
-            {
-                tag 'install-sgmlcatalog-deprecated', "$file:$.";
-            }
-            if (m,\binstall-info\b,) {
-                tag 'install-info-used-in-maintainer-script', "$file:$.";
-            }
-            if (   m,/var/lib/dpkg/status\b,
-                && $pkg ne 'base-files'
-                && $pkg ne 'dpkg') {
-                tag 'maintainer-script-uses-dpkg-status-directly', $file;
-            }
+
+            generic_check_bad_command($_, $file, $., $pkg, 1);
+
             if (m,$LEADIN(?:/usr/sbin/)?dpkg-divert\s,
                 && !/--(?:help|list|truename|version)/) {
                 if (/--local/) {
@@ -1129,6 +1086,33 @@ sub run {
 
 # -----------------------------------
 
+# try generic bad maintainer script command tagging
+sub generic_check_bad_command {
+    my ($line, $file, $lineno, $pkg, $findincatstring) = @_;
+    # try generic bad maintainer script command tagging
+  BAD_CMD:
+    foreach my $bad_cmd_tag ($BAD_MAINT_CMD->all) {
+        my $bad_cmd_data = $BAD_MAINT_CMD->value($bad_cmd_tag);
+        my $inscript = $bad_cmd_data->{'in_script'};
+        my $incat;
+        if ($file !~ m{$inscript}) {
+            next BAD_CMD;
+        }
+        $incat = $bad_cmd_data->{'in_cat_string'};
+        if ($incat == $findincatstring) {
+            my $regex = $bad_cmd_data->{'regexp'};
+            if ($line =~ m{$regex}) {
+                my $extrainfo = defined($1) ? "\'$1\'" : '';
+                my $inpackage = $bad_cmd_data->{'in_package'};
+                unless($pkg =~ m{$inpackage}) {
+                    tag $bad_cmd_tag, "$file:$.", $extrainfo;
+                }
+            }
+        }
+    }
+    return;
+}
+
 # Returns non-zero if the given file is not actually a shell script,
 # just looks like one.
 sub script_is_evil_and_wrong {
