@@ -29,7 +29,7 @@ use Carp qw(croak);
 use Scalar::Util qw(blessed);
 
 use Lintian::Path;
-use Lintian::Util qw(open_gz perm2oct normalize_pkg_path);
+use Lintian::Util qw(fail open_gz perm2oct normalize_pkg_path dequote_name);
 
 my %ROOT_INDEX_TEMPLATE = (
     'name'     => '',
@@ -45,6 +45,18 @@ my %ROOT_INDEX_TEMPLATE = (
     'time'     => '22:55:34',
 );
 
+# A cache for (probably) the 5 most common permission strings seen in
+# the wild.
+# It may seem obscene, but it has an extreme "hit-ratio" and it is
+# cheaper vastly than perm2oct.
+my %PERM_CACHE = map { $_ => perm2oct($_) } (
+    '-rw-r--r--', # standard (non-executable) file
+    '-rwxr-xr-x', # standard executable file
+    'drwxr-xr-x', # standard dir perm
+    'drwxr-sr-x', # standard dir perm with suid (lintian-lab on lintian.d.o)
+    'lrwxrwxrwx', # symlinks
+);
+
 =head1 NAME
 
 Lintian::Collect::Package - Lintian base interface to binary and source package data collection
@@ -170,9 +182,9 @@ Needs-Info requirements for using I<file_info>: file-info
 
 sub file_info {
     my ($self, $file) = @_;
-    if (exists $self->{file_info}) {
-        return ${$self->{file_info}{$file}}
-          if exists $self->{file_info}->{$file};
+    if (my $cache = $self->{file_info}) {
+        return ${$cache->{$file}}
+          if exists $cache->{$file};
         return;
     }
     my %interned;
@@ -211,6 +223,41 @@ sub file_info {
     return;
 }
 
+=item md5sums
+
+Returns a hashref mapping a FILE to its md5sum.  The md5sum is
+computed by Lintian during extraction and is not guaranteed to match
+the md5sum in the "md5sums" control file.
+
+Needs-Info requirements for using I<md5sums>: md5sums
+
+=cut
+
+sub md5sums {
+    my ($self) = @_;
+    return $self->{md5sums} if exists $self->{md5sums};
+    my $md5f = $self->lab_data_path('md5sums');
+    my $result = {};
+
+    # read in md5sums info file
+    open(my $fd, '<', $md5f);
+    while (my $line = <$fd>) {
+        chop($line);
+        next if $line =~ m/^\s*$/o;
+        $line =~ m/^(\\)?(\S+)\s*(\S.*)$/o
+          or fail "syntax error in $md5f info file: $line";
+        my ($zzescaped, $zzsum, $zzfile) = ($1, $2, $3);
+        if($zzescaped) {
+            $zzfile = dequote_name($zzfile);
+        }
+        $zzfile =~ s,^(?:\./)?,,o;
+        $result->{$zzfile} = $zzsum;
+    }
+    close($fd);
+    $self->{md5sums} = $result;
+    return $result;
+}
+
 =item index (FILE)
 
 Returns a L<path object|Lintian::Path> to FILE in the package.  FILE
@@ -315,26 +362,13 @@ sub _fetch_extracted_dir {
     return $dir;
 }
 
-# Strip an extra layer quoting in index file names and optionally
-# remove an initial "./" if any.
-#
-# sub _dequote_name Needs-Info none
-sub _dequote_name {
-    my ($name, $slsd) = @_;
-    $slsd = 1 unless defined $slsd; # Remove initial ./ by default
-    $name =~ s,^\.?/,, if $slsd;
-    $name =~ s/(\G|[^\\](?:\\\\)*)\\(\d{3})/"$1" . chr(oct $2)/ge;
-    $name =~ s/\\\\/\\/;
-    return $name;
-}
-
 # Backing method for index and others; this is not a part of the API.
 # sub _fetch_index_data Needs-Info none
 sub _fetch_index_data {
     my ($self, $field, $index, $indexown, $file) = @_;
-    if (exists $self->{$index}) {
-        return $self->{$field}->{$file}
-          if exists $self->{$index}->{$file};
+    if (my $cache = $self->{$field}) {
+        return $cache->{$file}
+          if exists $cache->{$file};
         return;
     }
     my $base_dir = $self->base_dir;
@@ -354,7 +388,16 @@ sub _fetch_index_data {
         my (%file, $perm, $owner, $name);
         ($perm,$owner,$file{size},$file{date},$file{time},$name)
           =split(' ', $_, 6);
-        $file{operm} = perm2oct($perm);
+        # This may appear to be obscene, but the call overhead of
+        # perm2oct is measurable on (e.g.) chromium-browser.  With
+        # the cache we go from ~1.5s to ~0.1s.
+        #   Of the 115363 paths here, only 306 had an "uncached"
+        # permission string (chromium-browser/32.0.1700.123-2).
+        if (exists($PERM_CACHE{$perm})) {
+            $file{operm} = $PERM_CACHE{$perm};
+        } else {
+            $file{operm} = perm2oct($perm);
+        }
         $file{type} = substr $perm, 0, 1;
 
         if ($num_idx) {
@@ -374,19 +417,19 @@ sub _fetch_index_data {
         $file{group} = 'root' if $file{group} eq '0';
 
         if ($name =~ s/ link to (.*)//) {
-            my $target = _dequote_name($1);
+            my $target = dequote_name($1);
             $file{type} = 'h';
             $file{link} = $target;
 
-            push @{$rhlinks{$target}}, _dequote_name($name);
+            push @{$rhlinks{$target}}, dequote_name($name);
         } elsif ($file{type} eq 'l') {
             ($name, $file{link}) = split ' -> ', $name, 2;
-            $file{link} = _dequote_name($file{link}, 0);
+            $file{link} = dequote_name($file{link}, 0);
         }
         # We store the name here, but will replace it later.  The
         # reason for storing it now is that we may need it during the
         # "hard-link fixup"-phase.
-        $file{'name'} = $name = _dequote_name($name);
+        $file{'name'} = $name = dequote_name($name);
 
         $idxh{$name} = \%file;
 
