@@ -24,7 +24,7 @@ use strict;
 use warnings;
 use autodie;
 
-use parent qw(Class::Accessor Clone);
+use parent qw(Class::Accessor::Fast Clone);
 
 use Carp qw(croak);
 
@@ -111,6 +111,33 @@ my @BIN_QUERY = ('package','version','architecture',);
 
 my @CHG_QUERY = ('source','version','architecture',);
 
+my %TYPE2INFO = (
+    'source' => {
+        'file-fields'  => \@SRC_FILE_FIELDS,
+        'file-header'  => SRCLIST_FORMAT,
+        'query-fields' => \@SRC_QUERY
+    },
+    'binary' => {
+        'file-fields'  => \@BIN_FILE_FIELDS,
+        'file-header'  => BINLIST_FORMAT,
+        'query-fields' => \@BIN_QUERY
+    },
+    'changes' => {
+        'file-fields'  => \@CHG_FILE_FIELDS,
+        'file-header'  => CHGLIST_FORMAT,
+        'query-fields' => \@CHG_QUERY
+    },
+    'GROUP' => {
+        'file-fields'  => undef, # Never written to disk
+        'file-header'  => undef, # Never written to disk
+        'query-fields' => \@GROUP_QUERY
+    },
+);
+
+# udeb behave exactly like binary, so share the underlying table
+#  \o/  ~50 bytes saved!
+$TYPE2INFO{'udeb'} = $TYPE2INFO{'binary'};
+
 =item new (TYPE[, GROUPING])
 
 Creates a new packages list for a certain type of packages.  This type
@@ -159,7 +186,7 @@ Returns the type of packages that this manifest has information about.
 
 # For some reason these getters seem to just return "undef", so they
 # have been added manually below.  (not sure if this is a usage error
-# or a bug in Class::Accessor)
+# or a bug in Class::Accessor::Fast)
 #Lintian::Lab::Manifest->mk_ro_accessors (qw(dirty type));
 
 sub dirty {
@@ -184,10 +211,8 @@ croak.
 
 sub read_list {
     my ($self, $file) = @_;
-    croak 'Cannot read a GROUP manifest' if $self->type eq 'GROUP';
-    my $header;
-    my $fields;
-    my $qf;
+    my $type = $self->type;
+    croak 'Cannot read a GROUP manifest' if $type eq 'GROUP';
 
     # Accept a scalar (as an "in-memory file") - write_list does the same
     if (my $r = ref $file) {
@@ -198,7 +223,8 @@ sub read_list {
         return unless -s $file;
     }
 
-    ($header, $fields, $qf) = $self->_type_to_fields;
+    my ($header, $fields, $qf)
+      = @{$TYPE2INFO{$type}}{'file-header', 'file-fields', 'query-fields'};
 
     $self->{'state'} = $self->_do_read_file($file, $header, $fields, $qf);
     $self->_mark_dirty(0);
@@ -218,8 +244,9 @@ On error, the contents of FILE are undefined.
 
 sub write_list {
     my ($self, $file) = @_;
-    croak 'Cannot write a GROUP manifest' if $self->type eq 'GROUP';
-    my ($header, $fields, undef) = $self->_type_to_fields;
+    my $type = $self->type;
+    croak 'Cannot write a GROUP manifest' if $type eq 'GROUP';
+    my ($header, $fields) = @{$TYPE2INFO{$type}}{'file-header', 'file-fields'};
     my $visitor;
 
     open(my $fd, '>', $file);
@@ -259,10 +286,10 @@ sub visit_all {
     my ($self, $visitor, @keys) = @_;
     my $root;
     my $type = $self->type;
-    my (undef, undef, $qf) = $self->_type_to_fields;
+    my $qf = $TYPE2INFO{$type}{'query-fields'};
 
     if (@keys) {
-        $root = $self->_do_get($self->{'state'}, @keys);
+        $root = $self->get(@keys);
         return unless $root;
         if (scalar @$qf == scalar @keys) {
             # If we are given an exact match, just visit that and
@@ -302,8 +329,19 @@ except for source packages
 
 sub get {
     my ($self, @keys) = @_;
-    @keys = $self->_make_keys($keys[0]) if scalar @keys == 1;
-    return $self->_do_get($self->{'state'}, @keys);
+    my $cur = $self->{'state'};
+    my $type = $self->type;
+    my $qf = $TYPE2INFO{$type}{'query-fields'};
+    my $max = scalar @$qf;
+    @keys = $self->_make_keys($keys[0])
+      if scalar(@keys) == 1 && ref($keys[0]);
+    $max = scalar @keys if scalar @keys < $max;
+    for (my $i = 0 ; $i < $max ; $i++) {
+        my $key = $keys[$i];
+        $cur = $cur->{$key};
+        return unless defined $cur;
+    }
+    return $cur;
 }
 
 sub _make_keys {
@@ -324,10 +362,11 @@ to ENTRY will not affect the data in the manifest.
 
 sub set {
     my ($self, $entry) = @_;
+    my $type = $self->type;
     croak 'Cannot alter a GROUP manifest directly'
-      if $self->type eq 'GROUP';
+      if $type eq 'GROUP';
     my %pdata;
-    my (undef, $fields, $qf) = $self->_type_to_fields;
+    my ($fields, $qf) = @{$TYPE2INFO{$type}}{'file-fields', 'query-fields'};
 
     # Copy the relevant fields - ensuring all fields are defined.
     foreach my $field (@$fields) {
@@ -339,7 +378,7 @@ sub set {
     }
     $self->_make_alias_fields(\%pdata);
 
-    $self->_do_set($self->{'state'}, $qf, \%pdata);
+    $self->_do_set($qf, \%pdata);
     $self->_mark_dirty(1);
     return 1;
 }
@@ -409,7 +448,7 @@ sub _do_delete {
     return 0 unless defined $lk;
 
     if (@keys) {
-        $hash = $self->_do_get($self->{'state'}, @keys);
+        $hash = $self->get(@keys);
     } else {
         $hash = $self->{'state'};
     }
@@ -529,7 +568,7 @@ sub _do_read_file {
             $entry->{$fields->[$i]} = $values[$i]//'';
         }
         $self->_make_alias_fields($entry);
-        $self->_do_set($root, $qf, $entry);
+        $self->_do_set($qf, $entry, $root);
     }
     close($fd);
     return $root;
@@ -559,24 +598,10 @@ sub _make_alias_fields {
     return;
 }
 
-sub _do_get {
-    my ($self, $root, @keys) = @_;
-    my $cur = $root;
-    my (undef, undef, $qf) = $self->_type_to_fields;
-    my $max = scalar @$qf;
-    $max = scalar @keys if scalar @keys < $max;
-    for (my $i = 0 ; $i < $max ; $i++) {
-        my $key = $keys[$i];
-        $cur = $cur->{$key};
-        return unless defined $cur;
-    }
-    return $cur;
-}
-
 sub _do_set {
-    my ($self, $root, $qf, $entry) = @_;
+    my ($self, $qf, $entry, $root) = @_;
     my $qfl = scalar @$qf - 1; # exclude the last element (see below)
-    my $cur = $root;
+    my $cur = $root // $self->{'state'};
     my $k;
 
     # Find the hash where the entry should be stored
@@ -596,44 +621,11 @@ sub _do_set {
     $k = $entry->{$qf->[$qfl]};
     $cur->{$k} = $entry;
     if (my $grouping = $self->{'grouping'}) {
-        $grouping->_do_set($grouping->{'state'}, \@GROUP_QUERY, $entry);
+        $grouping->_do_set(\@GROUP_QUERY, $entry);
     }
     return 1;
 }
 
-# Returns ($header, $fields, $qf) - their value is based on $self->type.
-# - $header is XXXLIST_FORMAT
-# - $fields is \@XXX_FILE_FIELDS
-# - $qf     is \@XXX_QUERY
-sub _type_to_fields {
-    my ($self) = @_;
-    my $header;
-    my $fields;
-    my $qf;
-    my $type = $self->{'type'};
-
-    if ($type eq 'source') {
-        $fields = \@SRC_FILE_FIELDS;
-        $qf = \@SRC_QUERY;
-        $header = SRCLIST_FORMAT;
-    } elsif ($type eq 'binary' || $type eq 'udeb') {
-        $fields = \@BIN_FILE_FIELDS;
-        $qf = \@BIN_QUERY;
-        $header = BINLIST_FORMAT;
-    } elsif ($type eq 'changes') {
-        $fields = \@CHG_FILE_FIELDS;
-        $qf = \@CHG_QUERY;
-        $header = CHGLIST_FORMAT;
-    } elsif ($type eq 'GROUP') {
-        $fields = undef; #N/A
-        $qf = \@GROUP_QUERY;
-        $header = undef; #N/A
-    } else {
-        croak "Unknown type $type";
-    }
-    return ($header, $fields, $qf);
-}
-
 # Self-recursing method powering visit_all
 sub _recurse_visit {
     my ($self, $hash, $visitor, $vdep, @keys) = @_;
