@@ -24,24 +24,33 @@ use warnings;
 
 use Carp qw(croak);
 
-use Lintian::Output qw(debug_msg);
-use Text_utils qw(dtml_to_html dtml_to_text split_paragraphs wrap_paragraphs);
-use Util qw(fail read_dpkg_control);
+use Lintian::Data;
+use Lintian::Tag::TextUtil
+  qw(dtml_to_html dtml_to_text split_paragraphs wrap_paragraphs);
+use Lintian::Tags qw();
+use Lintian::Util qw(fail);
 
 # The URL to a web man page service.  NAME is replaced by the man page
 # name and SECTION with the section to form a valid URL.  This is used
 # when formatting references to manual pages into HTML to provide a link
 # to the manual page.
 our $MANURL
-    = 'http://manpages.debian.net/cgi-bin/man.cgi?query=NAME&sektion=SECTION';
-
-# Stores the parsed tag information for all known tags.  Loaded the first
-# time new() is called.
-our %INFO;
+  = 'http://manpages.debian.net/cgi-bin/man.cgi?query=NAME&amp;sektion=SECTION';
 
 # Stores the parsed manual reference data.  Loaded the first time info()
 # is called.
-our %MANUALS;
+our $MANUALS
+  = Lintian::Data->new('output/manual-references', qr/::/,\&_load_manual_data);
+
+# Map severity/certainty levels to tag codes.
+our %CODES = (
+    pedantic  => { 'wild-guess' => 'P', possible => 'P', certain => 'P' },
+    wishlist  => { 'wild-guess' => 'I', possible => 'I', certain => 'I' },
+    minor     => { 'wild-guess' => 'I', possible => 'I', certain => 'W' },
+    normal    => { 'wild-guess' => 'I', possible => 'W', certain => 'W' },
+    important => { 'wild-guess' => 'W', possible => 'E', certain => 'E' },
+    serious   => { 'wild-guess' => 'E', possible => 'E', certain => 'E' },
+);
 
 =head1 NAME
 
@@ -49,7 +58,9 @@ Lintian::Tag::Info - Lintian interface to tag metadata
 
 =head1 SYNOPSIS
 
-    my $tag_info = Lintian::Tag::Info->new('some-tag');
+    my $cs = Lintian::CheckScript->new ("$ENV{'LINTIAN_ROOT'}/checks/",
+                                        'files');
+    my $tag_info = $cs->get_tag ('some-tag');
     print "Tag info is:\n";
     print $tag_info->description('text', '   ');
     print "\nTag info in HTML is:\n";
@@ -58,62 +69,47 @@ Lintian::Tag::Info - Lintian interface to tag metadata
 =head1 DESCRIPTION
 
 This module provides an interface to tag metadata as gleaned from the
-*.desc files describing the checks.  Currently, it is only used to format
-and return the tag description, but it provides a framework that can be
-used to retrieve other metadata about tags.
+*.desc files describing the checks.  It can be used to retrieve specific
+metadata elements or to format the tag description.
 
 =head1 CLASS METHODS
 
 =over 4
 
-=item new(TAG)
+=item new(HASH, SCRIPT_NAME, SCRIPT_TYPE)
 
-Creates a new Lintian::Tag::Info object for the given TAG.  Returns undef
-if the tag is unknown and throws an exception if there is a parse error
-reading the check description files or if TAG is not specified.
-
-The first time this method is called, all tag metadata will be loaded into
-a memory cache.  This information will be used to satisfy all subsequent
-Lintian::Tag::Info object creation, avoiding multiple file reads.  This
-however means that a running Lintian process will not notice changes to
-tag metadata on disk.
+Creates a new Lintian::Tag:Info.
 
 =cut
 
-# Load all tag data into the %INFO hash.  Called by new() if %INFO is
-# empty and hence called the first time new() is called.
-sub _load_tag_data {
-    my $root = $ENV{LINTIAN_ROOT} || '/usr/share/lintian';
-    for my $desc (<$root/checks/*.desc>) {
-        debug_msg(2, "Reading checker description file $desc ...");
-        my ($header, @tags) = read_dpkg_control($desc);
-        unless ($header->{'check-script'}) {
-            fail("missing Check-Script field in $desc");
-        }
-        for my $tag (@tags) {
-            unless ($tag->{tag}) {
-                fail("missing Tag field in $desc");
-            }
-            $tag->{info} = '' unless exists($tag->{info});
-            $INFO{$tag->{tag}} = $tag;
-        }
-    }
-}
-
-# Create a new object for the given tag.  We just use the hash created by
-# read_dpkg_control as the object, which means we slowly bless the objects
-# in %INFO as we return them.
 sub new {
-    my ($class, $tag) = @_;
-    croak('no tag specified') unless $tag;
-    _load_tag_data() unless %INFO;
-    if ($INFO{$tag}) {
-        my $self = $INFO{$tag};
-        bless($self, $class) unless ref($self) eq $class;
-        return $self;
-    } else {
-        return;
-    }
+    my ($class, $tag, $sn, $st) = @_;
+    my %copy;
+    my $self;
+    my $tagname;
+    croak 'no tag specified' unless $tag;
+    %copy = %$tag;
+    $self = \%copy;
+    croak 'Missing Tag field' unless $self->{'tag'};
+    $tagname = $self->{'tag'};
+    croak "Missing Severity field for $tagname" unless $self->{'severity'};
+    croak "Missing Certainty field for $tagname" unless $self->{'certainty'};
+    croak "Tag $tagname has invalid severity ($self->{'severity'}):"
+      . ' Must be one of '
+      . join(', ', @Lintian::Tags::SEVERITIES)
+      unless exists $CODES{$self->{'severity'}};
+    croak "Tag $tagname has invalid certainty ($self->{'certainty'}):"
+      . ' Must be one of '
+      . join(', ', @Lintian::Tags::CERTAINTIES)
+      unless exists $CODES{$self->{'severity'}}{$self->{'certainty'}};
+    $self->{'info'} = '' unless $self->{'info'};
+    $self->{'script'} = $sn;
+    $self->{'script-type'} = $st;
+    $self->{'effective-severity'} = $self->{severity};
+
+    bless $self, $class;
+
+    return $self;
 }
 
 =back
@@ -122,6 +118,32 @@ sub new {
 
 =over 4
 
+=item certainty()
+
+Returns the certainty of the tag.
+
+=cut
+
+sub certainty {
+    my ($self) = @_;
+    return $self->{certainty};
+}
+
+=item code()
+
+Returns the one-letter code for the tag.  This will be a letter chosen
+from C<E>, C<W>, C<I>, or C<P>, based on the tag severity, certainty, and
+other attributes (such as whether experimental is set).  This code will
+never be C<O> or C<X>; overrides and experimental tags are handled
+separately.
+
+=cut
+
+sub code {
+    my ($self) = @_;
+    return $CODES{$self->{'effective-severity'}}{$self->{certainty}};
+}
+
 =item description([FORMAT [, INDENT]])
 
 Returns the formatted description (the Info field) for a tag.  FORMAT must
@@ -136,26 +158,17 @@ formatted output.
 
 =cut
 
-# Load manual reference data into %MANUALS.  This information doesn't have
-# a single unique key and has multiple data values per key, so we don't
-# try to use the Lintian::Data interface.  Instead, we read a file
-# delimited by double colons.  We do use a path similar to Lintian::Data
-# to keep such files in the same general location.
+# Parse manual reference data from the data file.
 sub _load_manual_data {
-    my $root = $ENV{LINTIAN_ROOT} || '/usr/share/lintian';
-    open(REFS, '<', "$root/data/output/manual-references")
-        or fail("can't open $root/data/output/manual-references: $!");
-    local $_;
-    while (<REFS>) {
-        chomp;
-        next if /^\#/;
-        next if /^\s*$/;
-        next unless /^(.+?)::(.*?)::(.+?)::(.*?)$/;
-        my ($manual, $section, $title, $url) = split('::');
-        $MANUALS{$manual}{$section}{title} = $title;
-        $MANUALS{$manual}{$section}{url} = $url;
+    my ($key, $rawvalue, $pval) = @_;
+    my ($section, $title, $url) = split m/::/, $rawvalue, 3;
+    my $ret = undef;
+    if (not defined $pval) {
+        $ret = $pval = {};
     }
-    close REFS;
+    $pval->{$section}{title} = $title;
+    $pval->{$section}{url} = $url;
+    return $ret;
 }
 
 # Format a reference to a manual in the HTML that Lintian uses internally
@@ -164,12 +177,12 @@ sub _load_manual_data {
 # argument isn't a known manual.
 sub _manual_reference {
     my ($manual, $section) = @_;
-    _load_manual_data unless %MANUALS;
-    return '' unless exists $MANUALS{$manual}{''};
+    return '' unless $MANUALS->known($manual);
 
+    my $man = $MANUALS->value($manual);
     # Start with the reference to the overall manual.
-    my $title = $MANUALS{$manual}{''}{title};
-    my $url   = $MANUALS{$manual}{''}{url};
+    my $title = $man->{''}{title};
+    my $url   = $man->{''}{url};
     my $text  = $url ? qq(<a href="$url">$title</a>) : $title;
 
     # Add the section information, if present, and a direct link to that
@@ -181,10 +194,13 @@ sub _manual_reference {
     } elsif ($section and $section =~ /^[A-Z\d.]+$/) {
         $text .= " section $section";
     }
-    if ($section and exists $MANUALS{$manual}{$section}) {
-        my $title = $MANUALS{$manual}{$section}{title};
-        my $url   = $MANUALS{$manual}{$section}{url};
-        $text .= qq[ (<a href="$url">$title</a>)];
+    if ($section and exists $man->{$section}) {
+        my $sec_title = $man->{$section}{title};
+        my $sec_url   = $man->{$section}{url};
+        $text.=
+          $sec_url
+          ? qq[ (<a href="$sec_url">$sec_title</a>)]
+          : qq[ ($sec_title)];
     }
 
     return $text;
@@ -208,8 +224,13 @@ sub _format_reference {
             $text = qq(the <a href="$url">$ref</a> manual page);
         } elsif ($ref =~ m,^(ftp|https?)://,) {
             $text = qq(<a href="$ref">$ref</a>);
+        } elsif ($ref =~ m,^/,) {
+            $text = qq(<a href="file://$ref">$ref</a>);
+        } elsif ($ref =~ m,^#(\d+)$,) {
+            my $url = qq(http://bugs.debian.org/$1);
+            $text = qq(<a href="$url">$url</a>);
         }
-        push (@refs, $text) if $text;
+        push(@refs, $text) if $text;
     }
 
     # Now build an English list of the results with appropriate commas and
@@ -237,21 +258,26 @@ sub description {
     my $info = $self->{info};
     $info =~ s/\n[ \t]/\n/g;
     my @text = split_paragraphs($info);
+    my $severity = $self->severity;
+    my $certainty = $self->certainty;
+
     if ($self->{ref}) {
         push(@text, '', _format_reference($self->{ref}));
     }
-    if ($self->{severity} and $self->{certainty}) {
-        my $severity = $self->{severity};
-        my $certainty = $self->{certainty};
-        push(@text, '', "Severity: $severity, Certainty: $certainty");
+    push(@text, '', "Severity: $severity, Certainty: $certainty");
+    if ($self->{script} and $self->{'script-type'}){
+        my $script = $self->{script};
+        my $stype = $self->{'script-type'};
+        push(@text, '', "Check: $script, Type: $stype");
     }
-    if ($self->{experimental}) {
-        push(@text, '',
-             'This tag is marked experimental, which means that the code that'
-             . ' generates it is not as well-tested as the rest of Lintian'
-             . ' and might still give surprising results.  Feel free to'
-             . ' ignore experimental tags that do not seem to make sense,'
-             . ' though of course bug reports are always welcomed.');
+    if ($self->experimental) {
+        push(@text,
+            '',
+            'This tag is marked experimental, which means that the code that'
+              . ' generates it is not as well-tested as the rest of Lintian'
+              . ' and might still give surprising results.  Feel free to'
+              . ' ignore experimental tags that do not seem to make sense,'
+              . ' though of course bug reports are always welcome.');
     }
 
     # Format and return the output.
@@ -262,6 +288,84 @@ sub description {
     }
 }
 
+=item experimental()
+
+Returns true if this tag is experimental, false otherwise.
+
+=cut
+
+sub experimental {
+    my ($self) = @_;
+    return ($self->{experimental} and $self->{experimental} eq 'yes');
+}
+
+=item severity([$real])
+
+Returns the severity of the tag; if $real is a truth value
+the real (original) severity is returned, otherwise the
+effective severity is returned.
+
+See set_severity()
+
+=cut
+
+sub severity {
+    my ($self, $real) = @_;
+    return $self->{'effective-severity'} unless $real;
+    return $self->{severity};
+}
+
+=item set_severity($severity)
+
+Modifies the effective severity of the tag.
+
+=cut
+
+sub set_severity{
+    my ($self, $sev) = @_;
+    croak "Unknown severity $sev.\n" unless exists $CODES{$sev};
+    $self->{'effective-severity'} = $sev;
+    return;
+}
+
+=item script()
+
+Returns the check script corresponding to this tag.
+
+=cut
+
+sub script {
+    my ($self) = @_;
+    return $self->{script};
+}
+
+=item sources()
+
+Returns, as a list, the keywords for the sources of this tag from the
+references header.  This is only the top-level source, not any
+more-specific section or chapter.
+
+=cut
+
+sub sources {
+    my ($self) = @_;
+    return unless $self->{ref};
+    my @refs = split(',', $self->{ref});
+    @refs = map { s/^([\w-]+)\s.*/$1/; s/\(\S+\)$//; $_ } @refs;
+    return @refs;
+}
+
+=item tag()
+
+Returns the tag name.
+
+=cut
+
+sub tag {
+    my ($self) = @_;
+    return $self->{tag};
+}
+
 =back
 
 =head1 DIAGNOSTICS
@@ -352,4 +456,4 @@ lintian(1)
 # indent-tabs-mode: nil
 # cperl-indent-level: 4
 # End:
-# vim: syntax=perl sw=4 sts=4 ts=4 et shiftround
+# vim: syntax=perl sw=4 sts=4 sr et
