@@ -26,6 +26,7 @@ use autodie;
 
 use Carp qw(croak);
 use Cwd qw(abs_path);
+use Errno qw(ENOENT);
 use Exporter qw(import);
 
 use constant {
@@ -78,17 +79,22 @@ BEGIN {
           locate_helper_tool
           drain_pipe
           signal_number2name
+          dequote_name
+          load_state_cache
+          find_backlog
           $PKGNAME_REGEX),
         @{ $EXPORT_TAGS{constants} });
 }
 
-use Encode ();
-use FileHandle;
-use Lintian::Command qw(spawn);
 use Digest::MD5;
 use Digest::SHA;
+use Encode ();
+use FileHandle;
 use Scalar::Util qw(openhandle);
 
+use Lintian::Command qw(spawn);
+use Lintian::Relation::Version qw(versions_equal versions_comparator);
+
 =head1 NAME
 
 Lintian::Util - Lintian utility functions
@@ -306,7 +312,7 @@ Where %d is the line number of the issue and %s is one of:
 
 The field appeared twice in the paragraph.
 
-=item Continuation line outside a paragraph
+=item Continuation line outside a paragraph (maybe line %d should be " .")
 
 A continuation line appears outside a paragraph - usually caused by an
 unintended empty line before it.
@@ -385,13 +391,13 @@ sub visit_dpkg_paragraph {
     while (<$CONTROL>) {
         chomp;
 
-        if (/^\#/) {
+        if (substr($_, 0, 1) eq '#') {
             next unless $flags & DCTRL_NO_COMMENTS;
             die "syntax error at line $.: Comments are not allowed.\n";
         }
 
         # empty line?
-        if ((!$debconf && m/^\s*$/) or ($debconf && $_ eq '')) {
+        if ($_ eq '' || (!$debconf && m/^\s*$/)) {
             if ($open_section) { # end of current section
                 # pass the current section to the handler
                 $code->($section, $lines);
@@ -542,7 +548,8 @@ sub visit_dpkg_paragraph {
             $open_section
               or die join(q{ },
                 "syntax error at line $.:",
-                "Continuation line outside a paragraph.\n");
+                'Continuation line outside a paragraph (maybe line',
+                ($. - 1), qq{should be " .").\n});
 
             # Policy: Many fields' values may span several lines; in this case
             # each continuation line must start with a space or a tab.  Any
@@ -1098,10 +1105,12 @@ sub strip (_) { ## no critic (Subroutines::RequireFinalReturn)
     if (defined wantarray) {
         # perl 5.14 s///r would have been useful here.
         my ($arg) = @_;
-        $arg =~ s/^\s++|\s++$//g;
-        return $arg;
+        $arg =~ s/^\s++//;
+        # unpack 'A*' is faster than s/\s++$//
+        return unpack('A*', $arg);
     }
-    $_[0] =~ s/^\s++|\s++$//g;
+    $_[0] =~ s/^\s++//;
+    $_[0] = unpack('A*', $_[0]);
     # void context, so no return needed here.
 }
 
@@ -1120,12 +1129,10 @@ sub lstrip (_) { ## no critic (Subroutines::RequireFinalReturn)
 # prototype for default to $_
 sub rstrip (_) {  ## no critic (Subroutines::RequireFinalReturn)
     if (defined wantarray) {
-        # perl 5.14 s///r would have been useful here.
-        my ($arg) = @_;
-        $arg =~ s/\s++$//g;
-        return $arg;
+        # unpack 'A*' is faster than s/\s++$//
+        return unpack('A*', $_[0]);
     }
-    $_[0] =~ s/\s++$//;
+    $_[0] = unpack('A*', $_[0]);
     # void context, so no return needed here.
 }
 
@@ -1147,6 +1154,28 @@ sub check_path {
     return 0;
 }
 
+=item dequote_name(STR, REMOVESLASH)
+
+Strip an extra layer quoting in index file names and optionally
+remove an initial "./" if any.
+
+Remove initial ./ by default
+
+=cut
+
+sub dequote_name {
+    my ($name, $slsd) = @_;
+    $slsd = 1 unless defined $slsd;
+    $name =~ s,^\.?/,, if $slsd;
+    # Optimise for the case where the filename does not contain
+    # backslashes.  It is a fairly rare to see that in practise.
+    if (index($name, '\\') > -1) {
+        $name =~ s/(\G|[^\\](?:\\\\)*)\\(\d{3})/"$1" . chr(oct $2)/ge;
+        $name =~ s/\\\\/\\/g;
+    }
+    return $name;
+}
+
 =item signal_number2name(NUM)
 
 Given a number, returns the name of the signal (without leading
@@ -1341,6 +1370,70 @@ sub is_ancestor_of {
     return 0;
 }
 
+=item load_state_cache(STATE_DIR)
+
+[Reporting tools only] Load the state cache from STATE_DIR.
+
+=cut
+
+sub load_state_cache {
+    my ($state_dir) = @_;
+    my $state_file = "$state_dir/state-cache";
+    my $state = {};
+    my $fd;
+    require YAML::Any;
+    eval {open($fd, '<', $state_file);};
+    if (my $err = $@) {
+        if ($err->errno != ENOENT) {
+            # Present, but unreadable for some reason
+            die($err);
+        }
+        # Not present; presume empty
+        return $state;
+    }
+    eval {$state = YAML::Any::Load(slurp_entire_file($fd, 1));};
+    # Not sure what Load does in case of issues; perldoc YAML says
+    # very little about it.  Based on YAML::Error, I guess it will
+    # write stuff to STDERR and use die/croak, but it remains a
+    # guess.
+    if (my $err = $@) {
+        die("$state_file was invalid; please fix or remove it.\n$err");
+    }
+    $state //= {};
+
+    if (ref($state) ne 'HASH') {
+        die("$state_file was invalid; please fix or remove it.");
+    }
+    close($fd);
+    return $state;
+}
+
+=item find_backlog(LINTIAN_VERSION, STATE)
+
+[Reporting tools only] Given the current lintian version and the
+harness state, return a list of group ids that are part of the
+backlog.  The list is sorted based on what version of Lintian
+processed the package.
+
+=cut
+
+sub find_backlog {
+    my ($lintian_version, $state) = @_;
+    my (@list, @sorted);
+    for my $group_id (keys(%{$state})) {
+        my $last_version = '0';
+        if (exists($state->{$group_id}{'last-processed-by'})) {
+            $last_version = $state->{$group_id}{'last-processed-by'};
+        }
+        push(@list, [$group_id, $last_version])
+          if not versions_equal($last_version, $lintian_version);
+    }
+    @sorted = map { $_->[0] }
+      sort { versions_comparator($a->[1], $b->[1]) || $a->[0] cmp $b->[0] }
+      @list;
+    return @sorted;
+}
+
 =back
 
 =head1 SEE ALSO
